# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from traceotter.proto.v1 import traceotter_pb2 as traceotter_dot_proto_dot_v1_dot_traceotter__pb2


class TraceOtterServiceStub(object):
    def __init__(self, channel):
        self.IngestSpans = channel.unary_unary(
            "/traceotter.proto.v1.TraceOtterService/IngestSpans",
            request_serializer=traceotter_dot_proto_dot_v1_dot_traceotter__pb2.IngestSpansRequest.SerializeToString,
            response_deserializer=traceotter_dot_proto_dot_v1_dot_traceotter__pb2.IngestSpansResponse.FromString,
        )


class TraceOtterServiceServicer(object):
    def IngestSpans(self, request, context):
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")


def add_TraceOtterServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
        "IngestSpans": grpc.unary_unary_rpc_method_handler(
            servicer.IngestSpans,
            request_deserializer=traceotter_dot_proto_dot_v1_dot_traceotter__pb2.IngestSpansRequest.FromString,
            response_serializer=traceotter_dot_proto_dot_v1_dot_traceotter__pb2.IngestSpansResponse.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
        "traceotter.proto.v1.TraceOtterService", rpc_method_handlers
    )
    server.add_generic_rpc_handlers((generic_handler,))

