# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

_sym_db = _symbol_database.Default()

from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2  # noqa: F401


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n$traceotter/proto/v1/traceotter.proto\x12\x13traceotter.proto.v1\x1a\x1cgoogle/protobuf/struct.proto"\xd4\x02\n\x04Span\x12\x10\n\x08trace_id\x18\x01 \x01(\t\x12\x0f\n\x07span_id\x18\x02 \x01(\t\x12\x11\n\x04name\x18\x03 \x01(\tH\x00\x88\x01\x01\x12\x1b\n\x0estart_time_iso\x18\x04 \x01(\tH\x01\x88\x01\x01\x12\x1c\n\x0fstart_time_unix\x18\x05 \x01(\x01H\x02\x88\x01\x01\x12\x19\n\x0cend_time_iso\x18\x06 \x01(\tH\x03\x88\x01\x01\x12\x1a\n\rend_time_unix\x18\x07 \x01(\x01H\x04\x88\x01\x01\x12\x16\n\tparent_id\x18\x08 \x01(\tH\x05\x88\x01\x01\x12+\n\nattributes\x18\t \x01(\x0b\x32\x17.google.protobuf.StructB\x07\n\x05_nameB\x11\n\x0f_start_time_isoB\x12\n\x10_start_time_unixB\x0f\n\r_end_time_isoB\x10\n\x0e_end_time_unixB\x0c\n\n_parent_id">\n\x12IngestSpansRequest\x12(\n\x05spans\x18\x01 \x03(\x0b\x32\x19.traceotter.proto.v1.Span"Y\n\x13IngestSpansResponse\x12\x12\n\nrequest_id\x18\x01 \x01(\t\x12\x16\n\x0eaccepted_count\x18\x02 \x01(\x05\x12\x16\n\x0erejected_count\x18\x03 \x01(\x05\x32u\n\x11TraceOtterService\x12`\n\x0bIngestSpans\x12\'.traceotter.proto.v1.IngestSpansRequest\x1a(.traceotter.proto.v1.IngestSpansResponseb\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(
    DESCRIPTOR, "traceotter.proto.v1.traceotter_pb2", _globals
)
if not _descriptor._USE_C_DESCRIPTORS:
    DESCRIPTOR._loaded_options = None
    _globals["_SPAN"]._serialized_start = 92
    _globals["_SPAN"]._serialized_end = 432
    _globals["_INGESTSPANSREQUEST"]._serialized_start = 434
    _globals["_INGESTSPANSREQUEST"]._serialized_end = 496
    _globals["_INGESTSPANSRESPONSE"]._serialized_start = 498
    _globals["_INGESTSPANSRESPONSE"]._serialized_end = 587
    _globals["_TRACEOTTERSERVICE"]._serialized_start = 589
    _globals["_TRACEOTTERSERVICE"]._serialized_end = 706

